//
//  GLPointPair.m
//  ImageFunGL
//
//  Created by Tom Bernard on 5/22/07.
//  Copyright 2007 Bersearch Information Services. All rights reserved.
//

#import "GLPointPair.h"

NSString *glppPoint1Key = @"glppPoint1Key";
NSString *glppPoint2Key = @"glppPoint2Key";

extern double randomDoubleBetweenZeroOne();
extern double randomDoubleBetween(double ll, double ul);

@implementation GLPointPair

+ (id)randomPointPair
{
	GLdouble tdx = 0, tdy = 0, tdz = 0;
	
	GLPointWithColor *_point1, *_point2;
	_point1 = [GLPointWithColor pointWithX:randomDoubleBetween(-800.0, 800.0)
										 y:randomDoubleBetween(-800.0, 800.0)
										 z:randomDoubleBetween(-800.0, 800.0)
									   red:randomDoubleBetweenZeroOne()
									 green:randomDoubleBetweenZeroOne()
									  blue:randomDoubleBetweenZeroOne()
									 alpha:randomDoubleBetweenZeroOne()];
	
	[_point1 setDx:randomDoubleBetween(-1.0, 1.0)
				dy:randomDoubleBetween(-1.0, 1.0)
				dz:randomDoubleBetween(-1.0, 1.0)];
	
	
	_point2 = [GLPointWithColor pointWithX:randomDoubleBetween(-800.0, 800.0)
										 y:randomDoubleBetween(-800.0, 800.0)
										 z:randomDoubleBetween(-800.0, 800.0)];
	
	[_point2 setDx:randomDoubleBetween(-1.0, 1.0)
				dy:randomDoubleBetween(-1.0, 1.0)
				dz:randomDoubleBetween(-1.0, 1.0)];
	
	return [[[GLPointPair alloc] initWithPoint:_point1
									  andPoint:_point2] autorelease];
}



// designated initializer
- (id)initWithPoint:(GLPointWithColor *)_point1 andPoint:(GLPointWithColor *)_point2
{
	if (self = [super init])
	{
		// we ignore point2.color that is passed in; by setting point 2 before
		// setting point1, point2.color ends up as the complement of
		// point1.color
		
		[self setPoint2:_point2];
		[self setPoint1:_point1];
	}
	return self;
}


- (id)initWithCoder:(NSCoder *)decoder
{
	if (self = [super init])
	{
		[self setPoint2:[decoder decodeObjectForKey:glppPoint2Key]];
		[self setPoint1:[decoder decodeObjectForKey:glppPoint1Key]];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)encoder
{
	[encoder encodeObject:point1 forKey:glppPoint1Key];
	[encoder encodeObject:point2 forKey:glppPoint2Key];
}



- (void)render
{
	[point1 render];
	[point2 render];
}


- (void)animate;
{
	[point1 animate];
	[point2 animate];
}



- (NSColor *)complementaryColorForColor:(NSColor *)_color
{
	float hue, saturation, brightness, alpha;
	[_color getHue:&hue saturation:&saturation brightness:&brightness alpha:&alpha];
	hue += 0.5;
	if (hue > 1.0) hue -= 1.0;
	return [NSColor colorWithCalibratedHue:hue saturation:saturation brightness:brightness alpha:alpha];
}

- (GLPointWithColor *)point1
{
	return point1;
}

- (void)setPoint1:(GLPointWithColor *)_point1
{
	[_point1 retain];
	[point1 release];
	point1 = _point1;
	[point2 setColor:[self complementaryColorForColor:[point1 color]]];
}


- (GLPointWithColor *)point2
{
	return point2;
}

- (void)setPoint2:(GLPointWithColor *)_point2
{
	[_point2 retain];
	[point2 release];
	point2 = _point2;
	[point1 setColor:[self complementaryColorForColor:[point2 color]]];
}


@end
