//
//  GLPointWithColor.m
//  ImageFunGL
//
//  Created by Tom Bernard on 5/22/07.
//  Copyright 2007 Bersearch Information Services. All rights reserved.
//

#import "GLPointWithColor.h"
#import "ImageFunGLOpenGLView.h"

NSString *glpwcXKey			= @"glpwcXKey";
NSString *glpwcYKey			= @"glpwcYKey";
NSString *glpwcZKey			= @"glpwcZKey";

NSString *glpwcDxKey		= @"glpwcDxKey";
NSString *glpwcDyKey		= @"glpwcDyKey";
NSString *glpwcDzKey		= @"glpwcDzKey";


NSString *glpwcRedKey		= @"glpwcRedKey";
NSString *glpwcGreenKey		= @"glpwcGreenKey";
NSString *glpwcBlueKey		= @"glpwcBlueKey";
NSString *glpwcAlphaKey		= @"glpwcAlphaKey";


@implementation GLPointWithColor


// class methods

+ (id)pointWithX:(GLdouble)_x
			   y:(GLdouble)_y
			   z:(GLdouble)_z
			 red:(GLdouble)_red
		   green:(GLdouble)_green
			blue:(GLdouble)_blue
		   alpha:(GLdouble)_alpha
{
	return [[[GLPointWithColor alloc] initWithX:_x
											  y:_y
											  z:_z
											red:_red
										  green:_green
										   blue:_blue
										  alpha:_alpha] autorelease];
}

+ (id)pointWithX:(GLdouble)_x
			   y:(GLdouble)_y
			   z:(GLdouble)_z
		   color:(NSColor *)_color
{
	return [[[GLPointWithColor alloc] initWithX:_x
											  y:_y
											  z:_z
										  color:_color] autorelease];
}

+ (id)pointWithX:(GLdouble)_x
			   y:(GLdouble)_y
			   z:(GLdouble)_z
{
	return [[[GLPointWithColor alloc] initWithX:_x
											  y:_y
											  z:_z] autorelease];
}



// instance methods

// initializers

// designated initializer
- (id)initWithX:(GLdouble)_x
			  y:(GLdouble)_y
			  z:(GLdouble)_z
			red:(GLdouble)_red
		  green:(GLdouble)_green
		   blue:(GLdouble)_blue
		  alpha:(GLdouble)_alpha
{
	if (self = [super init])
	{
		x = _x;
		y = _y;
		z = _z;
		red = _red;
		green = _green;
		blue = _blue;
		alpha = _alpha;
		
		
		color = [[NSColor colorWithCalibratedRed:red
										   green:green
											blue:blue
										   alpha:alpha ] retain];
	}
	return self;
}

- (id)initWithX:(GLdouble)_x
			  y:(GLdouble)_y
			  z:(GLdouble)_z
		  color:(NSColor *)_color
{
	if (self = [super init])
	{
		x = _x;
		y = _y;
		z = _z;
		
		[self setColor:_color];
	}
	return self;
}

- (id)initWithX:(GLdouble)_x
			  y:(GLdouble)_y
			  z:(GLdouble)_z
{
	return [self initWithX:_x
						 y:_y
						 z:_z
					   red:0.0		// if no color specified, default to black
					 green:0.0
					  blue:0.0
					 alpha:1.0];
}

// <NSCoding>

- (id)initWithCoder:(NSCoder *)decoder
{
	if (self = [super init])
	{
		x		= [decoder decodeDoubleForKey:glpwcXKey];
		y		= [decoder decodeDoubleForKey:glpwcYKey];
		z		= [decoder decodeDoubleForKey:glpwcZKey];
		red		= [decoder decodeDoubleForKey:glpwcRedKey];
		green	= [decoder decodeDoubleForKey:glpwcGreenKey];
		blue	= [decoder decodeDoubleForKey:glpwcBlueKey];
		alpha	= [decoder decodeDoubleForKey:glpwcAlphaKey];
		
		color = [[NSColor colorWithCalibratedRed:red
										   green:green
											blue:blue
										   alpha:alpha ] retain];
	}
	return self;
}

- (void)encodeWithCoder:(NSCoder *)encoder
{
	[encoder encodeDouble:x			forKey:glpwcXKey];
	[encoder encodeDouble:y			forKey:glpwcYKey];
	[encoder encodeDouble:z			forKey:glpwcZKey];
	[encoder encodeDouble:red		forKey:glpwcRedKey];
	[encoder encodeDouble:green		forKey:glpwcGreenKey];
	[encoder encodeDouble:blue		forKey:glpwcBlueKey];
	[encoder encodeDouble:alpha		forKey:glpwcAlphaKey];
}



// render and animate

- (void)render
{
	glColor4d(red, green, blue, alpha);
	glVertex3d(x, y, z);
}


- (void)animate;
{
	/*
	x += dx;
	if (x < -edgeOfUniverse || x > edgeOfUniverse) dx *= -1.0;
	
	y += dy;
	if (y < -edgeOfUniverse || y > edgeOfUniverse) dx *= -1.0;
	
	z += dz;
	if (z < -edgeOfUniverse || z > edgeOfUniverse) dx *= -1.0;
	 */
}


// animation accessors

- (void)getDx:(GLdouble *)_dx dy:(GLdouble *)_dy dz:(GLdouble *)_dz
{
	*_dx = dx;
	*_dy = dy;
	*_dz = dz;
}

- (void)setDx:(GLdouble)_dx dy:(GLdouble)_dy dz:(GLdouble)_dz
{
	dx = _dx;
	dy = _dy;
	dz = _dz;
}

// accessors


- (GLdouble)x
{
	return x;
}

- (void)setX:(GLdouble)_x
{
	x = _x;
}


- (GLdouble)y
{
	return y;
}

- (void)setY:(GLdouble)_y
{
	y = _y;
}


- (GLdouble)z
{
	return z;
}

- (void)setZ:(GLdouble)_z
{
	z = _z;
}


- (GLdouble)red
{
	return red;
}

- (void)setRed:(GLdouble)_red
{
	red = _red;
}


- (GLdouble)green
{
	return green;
}

- (void)setGreen:(GLdouble)_green
{
	green = _green;
}


- (GLdouble)blue
{
	return blue;
}

- (void)setBlue:(GLdouble)_blue
{
	blue = _blue;
}


- (GLdouble)alpha
{
	return alpha;
}

- (void)setAlpha:(GLdouble)_alpha
{
	alpha = _alpha;
}


- (NSColor *)color
{
	return color;
}

- (void)setColor:(NSColor *)_color
{
	float _red, _green, _blue, _alpha;
	[_color retain];
	[color release];
	color = _color;
	[color getRed:&_red green:&_green blue:&_blue alpha:&_alpha];
	red = (GLdouble)_red;
	green = (GLdouble)_green;
	blue = (GLdouble)_blue;
	alpha = (GLdouble)_alpha;
}


@end

/*
	return ;
	 = ;
 */
