//
//  ImageFunGLOpenGLView.m
//  ImageFunGL
//
//  Created by Tom Bernard on 5/21/07.
//  Copyright Bersearch Information Services 2007. All rights reserved.
//

#import "ImageFunGLOpenGLView.h"
#import "GLPointPair.h"

NSString *ifgloglvPointPairArrayKey = @"ifgloglvPointPairArrayKey";
GLdouble edgeOfUniverse = 1000.0;

void reshapeGL(int x, int y, int width, int height)
{
	glViewport(x, y, width, height);
}



/*
double randomDoubleBetweenZeroOne()
  produces a random double between 0.0 and 1.0
 */

double randomDoubleBetweenZeroOne()
{
	return (double)random() / (double)(pow(2, 31) - 1);
}




/*
double randomDoubleBetween(double ll, double ul)
 produces a random double between ll and ul
 */

double randomDoubleBetween(double ll, double ul)
{
	return ll + (double)random() / (double)(pow(2, 31) - 1) * (ul - ll);
}

int randomIntBetweeen(int ll, int ul)
{
	return (int)randomDoubleBetween( (double)ll, (double)ul );
}




//		adapted from Assimmilator, by Brian Nenninger

static int coinflip() 
{
	return (randomDoubleBetweenZeroOne() > 0.5);
}

//		/adapted from Assimmilator, by Brian Nenninger



@implementation ImageFunGLOpenGLView

- (id)initWithFrame:(NSRect)frameRect
		pixelFormat:(NSOpenGLPixelFormat*)format
{	
    self = [super initWithFrame:frameRect pixelFormat:format];
	if (self)
	{
		
		pi = acos(-1.0);
		pi2 = 2.0 * pi;
		
		xangle = randomDoubleBetween(0.0, pi2);
		yangle = randomDoubleBetween(0.0, pi2);
		zangle = randomDoubleBetween(0.0, pi2);
		

		
		//		adapted from Assimmilator, by Brian Nenninger
		
		basedist = 20.0;
		
		dup = randomDoubleBetween(0.015, 0.030);
		drt = randomDoubleBetween(0.015, 0.030);
		
		if (coinflip()) dup = -dup;
		if (coinflip()) drt = -drt;
		
		maxrt = 4.0;
		maxup = 3.0;
		
		dxang = randomDoubleBetween(0.007, 0.012);
		dyang = randomDoubleBetween(0.007, 0.012);
		dzang = randomDoubleBetween(0.007, 0.012);
		
		if (coinflip()) dxang = -dxang;
		if (coinflip()) dyang = -dyang;
		if (coinflip()) dzang = -dzang;
		
		//		/adapted from Assimmilator, by Brian Nenninger
		
		
		
	}
	return self;
	
}

- (id)initWithFrame:(NSRect)frameRect
{
	// printf("-[%s initWithFrame:] ", [[self description] UTF8String]);
	// printfNSRect("frameRect", frameRect);
	
	
	NSOpenGLPixelFormatAttribute attrs[] =
	{
		NSOpenGLPFADoubleBuffer,
		NSOpenGLPFADepthSize, (NSOpenGLPixelFormatAttribute)32,
		(NSOpenGLPixelFormatAttribute)nil
	};
	
	NSOpenGLPixelFormat* pixFmt = [[[NSOpenGLPixelFormat alloc] initWithAttributes:attrs] autorelease];
	
	self = [self initWithFrame:frameRect pixelFormat:pixFmt];
	return self;
}

- (void)dealloc
{
	[pointPairArray release];
	
	[super dealloc];
	
}

- (void)delayedInit
{
	if(!inited)
	{
        [[self openGLContext] makeCurrentContext];
        
		swap_enabled = 1;
		[[self openGLContext] setValues:&swap_enabled forParameter:NSOpenGLCPSwapInterval];
		
		glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
        
		//		adapted from Assimmilator, by Brian Nenninger
		
		glMatrixMode(GL_PROJECTION);
		gluPerspective(  30.0,					// field of view in degrees
		 				4.0/3.0,				// aspect ratio
		 				0.1,  2*basedist);		// Z near , Z far
		
		
		//		/adapted from Assimmilator, by Brian Nenninger
		
		glMatrixMode(GL_MODELVIEW);
		
        inited = YES;
		[self reshape];
		
		// either load or generate the point-pair array
		
		NSArray *array = [imageFunGLDocument pointPairArray];
		if (array)
		{
			[self setPointPairArray:array];
		}
		else
		{
			[self generatePointPairs];
			[imageFunGLDocument setPointPairArray:pointPairArray];
		}
		
		[self startAnimation];
    }
	
}

- (void)reshape
{
    if(inited)
    {
		[[self openGLContext] makeCurrentContext];
		
		NSRect frame = [self frame];
		reshapeGL((int)frame.origin.x, (int)frame.origin.y, (int)frame.size.width, (int)frame.size.height);
		[self setNeedsDisplay:YES];
    }
}

- (void)drawRect:(NSRect)rect
{
	/*
	 printf("-[%s drawRect:] ", [[self description] UTF8String]);
	 printfNSRect("rect", rect);
	 NSWindow *myWindow = [[self superview] window];
	 printf("%s ", [[myWindow description] UTF8String]);
	 printfNSRect("frame ", [myWindow frame]);
	 */
	
	
	if (!inited) [self delayedInit];
	
	glClear(GL_COLOR_BUFFER_BIT);
	
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(-edgeOfUniverse, edgeOfUniverse, -edgeOfUniverse, edgeOfUniverse, -edgeOfUniverse, edgeOfUniverse);
	
	
	// postion camera at pre-calculated position
	
	//		lifted from Assimmilator, by Brian Nenninger
	
    gluLookAt(px, py, pz,
              px+fx, py+fy, pz+fz,
              ux, uy, uz);
	
	//		/lifted from Assimmilator, by Brian Nenninger
	
	glMatrixMode(GL_MODELVIEW);
	
	glBegin(GL_LINES);
	
	NSEnumerator *e = [pointPairArray objectEnumerator];
	GLPointPair *pointPair;
	while (pointPair = [e nextObject]) [pointPair render];
	
	glEnd();
	
	
	
	
    [[self openGLContext] flushBuffer];
	//  glutSwapBuffers();
	
}

- (void)startAnimation
{
	if (animationTimer) [animationTimer invalidate];
	animationTimer = [NSTimer scheduledTimerWithTimeInterval:0.016
													  target:self
													selector:@selector(animate:)
													userInfo:nil
													 repeats:YES];
}

- (void)calculateCameraPosition
{
	//		lifted from Assimmilator, by Brian Nenninger
	
	// set base camera position
	fx=-1.0; fy=0.0; fz=0.0;
	ux= 0.0; uy=0.0; uz=1.0;
	
	{
		double xtemp, ytemp, ztemp;
		// rotate forward vector
		// x
		xtemp = fx;
		ytemp = fy*cos(xangle)-fz*sin(xangle);
		ztemp = fz*cos(xangle)+fy*sin(xangle);
		fx=xtemp; fy=ytemp; fz=ztemp;
		// y
		xtemp = fx*cos(yangle)+fz*sin(yangle);
		ytemp = fy;
		ztemp = fz*cos(yangle)-fx*sin(yangle);
		fx=xtemp; fy=ytemp; fz=ztemp;
		// z
		xtemp = fx*cos(zangle)-fy*sin(zangle);
		ytemp = fy*cos(zangle)+fx*sin(zangle);
		ztemp = fz;
		fx=xtemp; fy=ytemp; fz=ztemp;
        
		// repeat for up vector
		xtemp = ux;
		ytemp = uy*cos(xangle)-uz*sin(xangle);
		ztemp = uz*cos(xangle)+uy*sin(xangle);
		ux=xtemp; uy=ytemp; uz=ztemp;
		// y
		xtemp = ux*cos(yangle)+uz*sin(yangle);
		ytemp = uy;
		ztemp = uz*cos(yangle)-ux*sin(yangle);
		ux=xtemp; uy=ytemp; uz=ztemp;
		// z
		xtemp = ux*cos(zangle)-uy*sin(zangle);
		ytemp = uy*cos(zangle)+ux*sin(zangle);
		ztemp = uz;
		ux=xtemp; uy=ytemp; uz=ztemp;
		
		// right is cross product of forward and up
		rx = fy*uz-fz*uy;
		ry = fz*ux-fx*uz;
		rz = fx*uy-fy*ux;
	}
	
	rtoff += drt;
	if ((drt>0 && rtoff>maxrt) || (drt<0 && rtoff<-maxrt)) drt = -drt;
	
	upoff += dup;
	if ((dup>0 && upoff>maxup) || (dup<0 && upoff<-maxup)) dup = -dup;
	
	px = -basedist*fx + upoff*ux + rtoff*rx;
	py = -basedist*fy + upoff*uy + rtoff*ry;
	pz = -basedist*fz + upoff*uz + rtoff*rz;
	
	//		/lifted from Assimmilator, by Brian Nenninger
	
	
}

- (void)animate:(NSTimer *)timer
{
	//		adapted from Assimmilator, by Brian Nenninger
	
	// increment angles
	xangle += dxang; if (xangle>pi2) xangle-=pi2;
	yangle += dyang; if (yangle>pi2) yangle-=pi2;
	zangle += dzang; if (zangle>pi2) zangle-=pi2;
	
	[self calculateCameraPosition];		// better to do this now instead of during -drawRect:  - TB
	
	//		/adapted from Assimmilator, by Brian Nenninger
	
	[self animatePointPairs];
	
	
	[self setNeedsDisplay:YES];
}

- (void)animatePointPairs
{
	NSEnumerator *e = [pointPairArray objectEnumerator];
	GLPointPair *pointPair;
	while (pointPair = [e nextObject]) [pointPair animate];
}

- (void)generatePointPairs
{
	NSMutableArray *_array = [[NSMutableArray alloc] initWithCapacity:0];
	
	int count = randomIntBetweeen(10, 50);
	while (count)
	{
		[_array addObject:[GLPointPair randomPointPair]];
		count--;
	}
	[self setPointPairArray:[NSArray arrayWithArray:_array]];
	[_array release];
}

- (NSArray *)pointPairArray
{
	return pointPairArray;
}

- (void)setPointPairArray:(NSArray *)_array
{
	[_array retain];
	[pointPairArray release];
	pointPairArray = _array;
}

@end
